package be.iminds.ilabt.jfed.testing.tests.highlevel;

import com.fasterxml.jackson.databind.ObjectMapper;
import io.dropwizard.jackson.Jackson;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.Map;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;

import static org.mockito.Mockito.mock;

public class GuiLogicTestConfigTest {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

    @Test
    public void testDefaults() throws Exception {
        GuiLogicTestConfig guiLogicTestConfig = new GuiLogicTestConfig();
        String jsonString = guiLogicTestConfig.stringValue();

        String prettyJsonString = MAPPER.writerWithDefaultPrettyPrinter().writeValueAsString(guiLogicTestConfig);
        System.out.println("Default GuiLogicTestConfig JSON: "+prettyJsonString+"");

        assertSameJsonObjects(jsonString, "{}");
        assertSameJsonObjects(prettyJsonString, "{}");

        checkDefaults(guiLogicTestConfig);
    }

    @Test
    public void testDefaultsRecover() throws Exception {
        String jsonString = "{}";
        GuiLogicTestConfig guiLogicTestConfig = MAPPER.readValue(jsonString, GuiLogicTestConfig.class);

        String prettyJsonString = MAPPER.writerWithDefaultPrettyPrinter().writeValueAsString(guiLogicTestConfig);
        System.out.println("Recovered Default GuiLogicTestConfig JSON: "+prettyJsonString+"");

        assertSameJsonObjects(prettyJsonString, jsonString);

        checkDefaults(guiLogicTestConfig);
    }

    private static String nonDefaultJsonString = "{\n" +
            "\t\"amConnectionProxy\": {\n" +
            "\t\t\"mode\": \"USER_AUTHORITY\"\n" +
            "\t},\n" +
            "\t\"ansibleTest\": {\n" +
            "\t\t\"enabled\": true,\n" +
            "\t\t\"debug\": true,\n" +
            "\t\t\"playbookExe\": \"ansible-playbook\",\n" +
            "\t\t\"playbookUrl\": \"http://www.gpolab.bbn.com/experiment-support/HelloGENI/hellogeni_ansible.yml\",\n" +
            "\t\t\"playbookContent\": null,\n" +
            "\t\t\"failureRegex\": \"failed=[^0]\",\n" +
            "\t\t\"warningRegex\": \"unreachable=[^0]\",\n" +
            "\t\t\"successRegex\": \"ok=[^0]\"\n" +
            "\t}\n" +
            "}\n";
    private static String nonDefaultMinimalJsonString = "{\n" +
            "\t\"amConnectionProxy\": {\n" +
            "\t\t\"mode\": \"USER_AUTHORITY\"\n" +
            "\t},\n" +
            "\t\"ansibleTest\": {\n" +
            "\t\t\"enabled\": true,\n" +
            "\t\t\"debug\": true,\n" +
            "\t\t\"playbookUrl\": \"http://www.gpolab.bbn.com/experiment-support/HelloGENI/hellogeni_ansible.yml\"\n" +
            "\t}\n" +
            "}\n";

    @Test
    public void testNonDefaultsRecover() throws Exception {
        GuiLogicTestConfig guiLogicTestConfig = MAPPER.readValue(nonDefaultJsonString, GuiLogicTestConfig.class);

        String prettyJsonString = MAPPER.writerWithDefaultPrettyPrinter().writeValueAsString(guiLogicTestConfig);
        System.out.println("Recovered Default GuiLogicTestConfig JSON: "+prettyJsonString+"");

        assertSameJsonObjects(prettyJsonString, nonDefaultMinimalJsonString);

        checkNonDefault(guiLogicTestConfig);
    }

    @Test
    public void testNonDefaultsRecoverMinimal() throws Exception {
        GuiLogicTestConfig guiLogicTestConfig = MAPPER.readValue(nonDefaultMinimalJsonString, GuiLogicTestConfig.class);

        String prettyJsonString = MAPPER.writerWithDefaultPrettyPrinter().writeValueAsString(guiLogicTestConfig);
        System.out.println("Recovered Default GuiLogicTestConfig JSON: "+prettyJsonString+"");

        assertSameJsonObjects(prettyJsonString, nonDefaultMinimalJsonString);

        checkNonDefault(guiLogicTestConfig);
    }


    private void checkNonDefault(GuiLogicTestConfig guiLogicTestConfig) {
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getMode(), is(equalTo(GuiLogicTestConfig.ProxyMode.USER_AUTHORITY)));
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getHostName(), is(nullValue()));
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getHostPort(), is(equalTo(22)));
        assertThat(guiLogicTestConfig.getAnsibleTest().isEnabled(), is(equalTo(true)));
        assertThat(guiLogicTestConfig.getAnsibleTest().getPlaybookUrl(), is(equalTo("http://www.gpolab.bbn.com/experiment-support/HelloGENI/hellogeni_ansible.yml")));
        assertThat(guiLogicTestConfig.getAnsibleTest().getPlaybookContent(), is(nullValue()));
        assertThat(guiLogicTestConfig.getAnsibleTest().getDebug(), is(equalTo(true)));
        assertThat(guiLogicTestConfig.getAnsibleTest().getFailureRegex(), is(equalTo("failed=[^0]")));
        assertThat(guiLogicTestConfig.getAnsibleTest().getWarningRegex(), is(equalTo("unreachable=[^0]")));
        assertThat(guiLogicTestConfig.getAnsibleTest().getSuccessRegex(), is(equalTo("ok=[^0]")));
        assertThat(guiLogicTestConfig.getAnsibleTest().getProxy().getMode(), is(equalTo(GuiLogicTestConfig.ProxyMode.OFF)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().isEnabled(), is(equalTo(true)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().getProxy().getMode(), is(equalTo(GuiLogicTestConfig.ProxyMode.OFF)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().getForceIpVersion(), is(nullValue()));

        assertThat(guiLogicTestConfig.getResources(), hasSize(1));
        GuiLogicTestConfig.Resource resource = guiLogicTestConfig.getResources().get(0);
//        assertThat(resource.getProvision().getProvisionExpireTimeMin(), is(equalTo(25l)));
        assertThat(resource.getRequestRSpec().getjFedResourceClass(), is(nullValue()));
        assertThat(resource.getRequestRSpec().getSource(), is(equalTo(GuiLogicTestConfig.RequestRSpecSource.GENERATE_USING_JFED)));
        assertThat(resource.getRequestRSpec().getUseNitosLease(), is(equalTo(false)));
        assertThat(resource.getSlice().getFailIfNoProject(), is(equalTo(false)));
        assertThat(resource.getSlice().getProject(), is(nullValue()));
        assertThat(resource.getWaitForReady().getSkipFinalManifestCheck(), is(equalTo(false)));
        assertThat(resource.getWaitForReady().getExtraWaitTimeSeconds(), is(equalTo(0l)));
    }

    private void checkDefaults(GuiLogicTestConfig guiLogicTestConfig) {
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getMode(), is(equalTo(GuiLogicTestConfig.ProxyMode.OFF)));
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getHostName(), is(nullValue()));
        assertThat(guiLogicTestConfig.getAmConnectionProxy().getHostPort(), is(equalTo(22)));
        assertThat(guiLogicTestConfig.getAnsibleTest().isEnabled(), is(equalTo(false)));
        assertThat(guiLogicTestConfig.getAnsibleTest().getDebug(), is(equalTo(false)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().isEnabled(), is(equalTo(true)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().getProxy().getMode(), is(equalTo(GuiLogicTestConfig.ProxyMode.OFF)));
        assertThat(guiLogicTestConfig.getNodeLoginTest().getForceIpVersion(), is(nullValue()));

        assertThat(guiLogicTestConfig.getResources(), hasSize(1));
        GuiLogicTestConfig.Resource resource = guiLogicTestConfig.getResources().get(0);
//        assertThat(resource.getProvision().getProvisionExpireTimeMin(), is(equalTo(25l)));
        assertThat(resource.getRequestRSpec().getjFedResourceClass(), is(nullValue()));
        assertThat(resource.getRequestRSpec().getSource(), is(equalTo(GuiLogicTestConfig.RequestRSpecSource.GENERATE_USING_JFED)));
        assertThat(resource.getRequestRSpec().getUseNitosLease(), is(equalTo(false)));
        assertThat(resource.getSlice().getFailIfNoProject(), is(equalTo(false)));
        assertThat(resource.getSlice().getProject(), is(nullValue()));
        assertThat(resource.getWaitForReady().getSkipFinalManifestCheck(), is(equalTo(false)));
        assertThat(resource.getWaitForReady().getExtraWaitTimeSeconds(), is(equalTo(0l)));
    }

    private void assertSameJsonObjects(String a, String b) throws IOException {
        Map am = MAPPER.readValue(a, Map.class);
        Map bm = MAPPER.readValue(b, Map.class);
        assertThat("json differs: "+a+"\n\n-----------\n\n"+b, am, is(equalTo(bm)));
    }
}
